#include <amxmodx>
#include <amxmisc>
#include <ze_core>

// Cvars.
new g_iJumpCount,
	g_iParachute,
	g_iMultiJumps,
	bool:g_bSlowDown,
	Float:g_flFallSpeed

// Variables.
new g_bitsHasParachute,
	g_bitsHasMultiJumps

// Arrays.
new g_iJumps[MAX_PLAYERS+1],
	g_iMaxJumps[MAX_PLAYERS+1]

public plugin_natives()
{
	register_library("ze_smooth_move")
	register_native("ze_set_user_multijump", "__native_set_user_multijump")
	register_native("ze_set_user_parachute", "__native_set_user_parachute")
}

public plugin_init()
{
	// Load Plug-In.
	register_plugin("[ZE] Addon: Smooth Move", "1.2.1", ZE_AUTHORS)

	// Hook Chains.
	RegisterHookChain(RG_PM_AirMove, "fw_PM_AirMove")
	RegisterHookChain(RG_CBasePlayer_Jump, "fw_PlayerJump")
	RegisterHookChain(RG_CBasePlayer_Jump, "fw_PlayerJump_Post", 1)

	// Cvars.
	bind_pcvar_num(register_cvar("ze_slowdown_enable", "1"), g_bSlowDown)
	bind_pcvar_num(register_cvar("ze_multijump_enable", "1"), g_iMultiJumps)
	bind_pcvar_num(register_cvar("ze_multijump_count", "2"), g_iJumpCount)
	bind_pcvar_num(register_cvar("ze_parachute_enable", "1"), g_iParachute)
	bind_pcvar_float(register_cvar("ze_parachute_fallspeed", "90.0"), g_flFallSpeed)
}

public client_disconnected(id, bool:drop, message[], maxlen)
{
	g_iJumps[id] = 0
	g_iMaxJumps[id] = 0

	flag_unset(g_bitsHasParachute, id)
	flag_unset(g_bitsHasMultiJumps, id)
}

public fw_PlayerJump(id)
{
	if (!is_user_alive(id))
		return HC_CONTINUE

	static bitsFlags;bitsFlags = get_entvar(id, var_flags)

	if (bitsFlags & FL_WATERJUMP || get_entvar(id, var_waterlevel) > 0 || !(get_member(id, m_afButtonPressed) & IN_JUMP))
		return HC_CONTINUE

	if (bitsFlags & FL_ONGROUND)
	{
		g_iJumps[id] = 0
		return HC_CONTINUE
	}

	if (!flag_get_boolean(g_bitsHasMultiJumps, id) && !check_PlayerTeam(id, g_iMultiJumps))
		return HC_CONTINUE

	static iMaxJumps
	if (!(iMaxJumps = g_iMaxJumps[id]))
		iMaxJumps = g_iJumpCount

	// Maximum Jumps exceeded?
	if (++g_iJumps[id] >= iMaxJumps)
		return HC_CONTINUE

	static Float:vSpeed[3]
	get_entvar(id, var_velocity, vSpeed)
	vSpeed[2] = 268.328157
	set_entvar(id, var_velocity, vSpeed)

	vSpeed = NULL_VECTOR
	return HC_SUPERCEDE
}

public fw_PlayerJump_Post(id)
{
	if (g_bSlowDown || !is_user_alive(id))
		return

	set_entvar(id, var_fuser2, 0.0)
}

public fw_PM_AirMove(id)
{
	if (!g_iParachute)
		return

	if (!is_user_alive(id) || !(get_entvar(id, var_button) & IN_USE) || get_entvar(id, var_waterlevel) > 0)
		return

	if (!flag_get_boolean(g_bitsHasParachute, id) && !check_PlayerTeam(id, g_iParachute))
		return

	static Float:vSpeed[3]
	get_entvar(id, var_velocity, vSpeed)

	if (vSpeed[2] < 0.0)
	{
		vSpeed[2] = (vSpeed[2] + 40.0 < -100.0) ? vSpeed[2] + 40.0 : -g_flFallSpeed

		set_movevar(mv_gravity, 80.0)
		set_pmove(pm_velocity, vSpeed)
		set_entvar(id, var_sequence, ACT_WALK)
		set_entvar(id, var_gaitsequence, ACT_IDLE)
	}

	vSpeed = NULL_VECTOR
}

/**
 * -=| Function |=-
 */
check_PlayerTeam(id, cvar = 0)
{
	switch (cvar)
	{
		case 1:
		{
			return 1
		}
		case 2: // Human only
		{
			if (!ze_is_user_zombie(id))
				return 1
		}
		case 3: // Zombie only.
		{
			if (ze_is_user_zombie(id))
				return 1
		}
	}

	return 0
}

public ze_roundend(iWinTeam)
{
	g_bitsHasParachute = 0
	g_bitsHasMultiJumps = 0
}

/**
 * -=| Natives |=-
 */
public __native_set_user_multijump(const plugin_id, const num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return false
	}

	if (get_param(2))
		flag_set(g_bitsHasMultiJumps, id), g_iMaxJumps[id] = get_param(3)
	else
		flag_unset(g_bitsHasMultiJumps, id), g_iMaxJumps[id] = 0
	return true
}

public __native_set_user_parachute(const plugin_id, const num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return false
	}

	if (get_param(2))
		flag_set(g_bitsHasParachute, id)
	else
		flag_unset(g_bitsHasParachute, id)
	return true
}